@@ -141,7 +141,7 @@ struct hash;
  * @param err   0 if success, otherwise errorcode
  * @param hdr   DNS Header
  * @param ansl  List of Answer records
- * @param authl List of Authoritive records
+ * @param authl List of Authoritative records
  * @param addl  List of Additional records
  * @param arg   Handler argument
  */
@@ -201,11 +201,13 @@ struct dnsc_conf {
 	uint32_t conn_timeout;  /* in [ms] */
 	uint32_t idle_timeout;  /* in [ms] */
 	uint32_t cache_ttl_max; /* in [s] 0 for disabled */
+	bool     getaddrinfo;   /* use getaddrinfo (by default disabled) */
 };
 
 int  dnsc_alloc(struct dnsc **dcpp, const struct dnsc_conf *conf,
 		const struct sa *srvv, uint32_t srvc);
 int  dnsc_conf_set(struct dnsc *dnsc, const struct dnsc_conf *conf);
+void dnsc_conf_set_timeout(struct dnsc *dnsc, uint32_t connect, uint32_t idle);
 int  dnsc_srv_set(struct dnsc *dnsc, const struct sa *srvv, uint32_t srvc);
 int  dnsc_query(struct dns_query **qp, struct dnsc *dnsc, const char *name,
 		uint16_t type, uint16_t dnsclass,
@@ -220,6 +222,8 @@ int  dnsc_notify(struct dns_query **qp, struct dnsc *dnsc, const char *name,
 		 dns_query_h *qh, void *arg);
 void dnsc_cache_flush(struct dnsc *dnsc);
 void dnsc_cache_max(struct dnsc *dnsc, uint32_t max);
+void dnsc_getaddrinfo(struct dnsc *dnsc, bool active);
+bool dnsc_getaddrinfo_enabled(struct dnsc *dnsc);
 
 
 /* DNS System functions */
